/*********************************************************\
*  (C) 2005 VEMS Group                                    *
\*********************************************************/

/* Type declarations global to everything */

#ifndef GLOBAL_H
#define GLOBAL_H

#include <inttypes.h>
/*
#include <avr/pgmspace.h>
#include <avr/interrupt.h>
*/

/*
 * This will force all functions declared with INLINE to be inlined. No object
 * code will ever be emitted.
 */
#ifndef INLINE
#define INLINE extern __inline__
#endif

struct config_t {/* must less size than 249byte*/
	/* 0x00 EGT */
	uint8_t egt_cal;			/* always appr. 153 */
	uint8_t egt_offset;			/* EGT offset; typical 64..192 (default: 192) */

	/* 0x02 WBO2 baselines */
	uint8_t wbo2_pump_pw_zero;		/* wbo2, pump zero dutycycle (default: 65) [5V/256] */
	uint8_t wbo2_nernstdc_target;		/* wbo2, target nernstDC [5V/256] */

	/* 0x04 WBO2 measurement and controller */
	uint8_t wbo2_ri_target;			/* wbo2, target Ri (pulse amplitude:C3)[5V/4096] */
	uint8_t wbo2_heater_pid_kp;		/* wbo2, heater pid kp */
	uint8_t wbo2_heater_pid_ki;		/* wbo2, heater pid ki */
	uint8_t wbo2_heater_pid_kd;		/* wbo2, heater pid kd */
	uint8_t wbo2_heater_pid_ilimit;		/* wbo2, heater pid integral limit */

	uint8_t wbo2_pump_pid_kp;		/* wbo2, pump pid kp */
	uint8_t wbo2_pump_pid_ki;		/* wbo2, pump pid ki */
	uint8_t wbo2_pump_pid_kd;		/* wbo2, pump pid kd */
	uint8_t wbo2_pump_pid_ilimit;		/* wbo2, pump pid integral limit */

	uint8_t wbo2_ri_confidence_scale;	/* wbo2, Ri confidence scaling factor (s^2 < 255/scale) */

	/* 0x0E WBO2 heater */
	uint8_t wbo2_warmup_ramp;		/* wbo2, warmup ramp speed [3f=0.2V/sec] */
	uint8_t wbo2_abs_limit;			/* wbo2, heater absolute limit [2a=12V ff=30V] */
	uint8_t wbo2_limit_maxt;		/* wbo2, max time spent at abs_limit [262msec] */
	uint8_t wbo2_fallback;			/* wbo2, fallback V after maxt [1c=10V ff=30V] */
	uint8_t wbo2_retry_t;			/* wbo2, retry control after [4sec] */

	/* 0x13 vout conf */
	uint8_t vout_conf;			/* vout conf 0:EGT,1:NBO2,2:Ip,4:lambda_curve,5:highslope;32..255:lambda linear out (156 = 3V/lamdba) */

	/* 0x14 Display state and scaling */
	uint8_t afrmult;			/* lambda -> afr multiplier (147) */
	uint8_t disp_state;			/* last known display state */
	uint8_t disp_dim;			/* display dimming */
	uint8_t disp_type;			/* 0,2,4 for normal, HUD1, HUD2 */

	/* 0x18 Boostcontrol */
	uint8_t boost_channel;

	/* 0x19 RPM */
	uint8_t rpm_toothcnt;			/* 4cyl:2 6cyl:3 60-2:58 */
	uint8_t rpm_multiplier;			/* currently unused (useful for wheelspeed ?) */

	/* 0x1B Display */
	uint8_t display_mask;
	uint8_t segment0;	/* 0xA5=165 means AIM data */
	uint8_t segment1;
	uint8_t segment2;
	uint8_t segment3;	/* 0xA5=165 means AIM data */
	uint8_t segment4;
	uint8_t segment5;
	uint8_t segment6;

	/* 0x23 Voltage output curve */
	uint8_t vout_curve[17];

	/* 0x34 Button */
	uint8_t button_threshold; /* 0:disabled ; 128: 2.5V */

	/* 0x35 Analog input analin_curve[adc/16] * analin_cal + analin_offset
	* usable for MAP, FP, MAT, etc.
	* 0..2024 maxrange (analin_cal = analin_curve[] = FF)
	* analin_mode (use 0 for reserved bits):
	* bit7 =0 reserved for bypass lookup and interpolation NOT IMPLEMENTED
	* bit6:4 =0 reserved analin_cal_exp for analin_cal exponent NOT IMPLEMENTED
	* bit3 =0 reserved
	* bit2:0 dot placement 123.4 12.34 1.234 1234 ... 1234.
	* MSB digit can be masked out already with a constant anything (possibly blank)
	*/

	uint8_t analin_curve[17]; /* detailed curve or offset and calibration (when linear) */
	uint8_t analin_cal; /* curve scale */
	uint8_t analin_offset; /* nonnegative offset */
	uint8_t analin_mode;	/* dot position bit2:0=0..7 (7=after last digit) bit7:3=0 reserved */

	/* 0x49 boost_out control */
	uint8_t out_minrpm; /* 64 RPM */
	uint8_t out_maxrpm; /* 64 RPM */
	uint8_t out_minegt; /* 0:-50C, 6:-2C, unit is [8C] */
	uint8_t out_maxegt; /* 0:-50C, 6:-2C, unit is [8C] */
	uint8_t hack;
	uint8_t out_maxain; /* units is 8 whatever */

    uint8_t aim_channel0;
	uint8_t aim_a0;
	uint16_t aim_b0;
    uint8_t aim_channel1;
	uint8_t aim_a1;
	uint16_t aim_b1;
    uint8_t aim_channel2;
	uint8_t aim_a2;
	uint16_t aim_b2;
    uint8_t aim_channel3;
	uint8_t aim_a3;
	uint16_t aim_b3;
	uint8_t aim_dot; /* top and bottom dotpos.Packed in bits 1:0 3:2 5:4 and 7:6 */
	uint8_t display_mode[8]; /* Display mode, top 4 bits 0xF0, bottom 4 bits 0x0F */
	uint8_t units; /* units:bit7=AiMdis/en bit3=lambda/afr bit2=kpa/psi bit1=kmh/mph bit0=C/F */
	uint8_t out_minain; /* units is 8 whatever */

	uint8_t boostcontrol_kp; /* gas mixture lambda / boost control P term of PI */
	uint8_t boostcontrol_ki; /* gas mixture lambda / boost control I term of PI */
	uint8_t boostcontrol_kd; /* gas mixture lambda / boost control I term of PI */
	uint8_t boostcontrol_ilimit; /* gas mixture lambda / boost control pid integral limit */
	uint8_t boostcontrol_mode; /* bit 0..1: strategy: 00:lambda control, 01: boost control , 10: idle control */
	uint8_t boosttarget[8]; /* 128 is 1.2; range: 0.7 .. 1.7 (practical max is 1.5) if lambdatarget, but we use only boosttarget[0]
				*  0 .. 255 if boosttarget with analog input
				* boosttargetrpm is fix 2400... with 700rpm step */
	uint8_t boosttarget_refdc[8]; /* 128 is 1.2; range: 0.7 .. 1.7 (practical max is 1.5) */
	uint8_t boostrpmbins[8]; /* RPM bins for boosttarget_refdc and boosttarget */
    uint8_t aim_channel4;
	uint8_t aim_a4;
	uint16_t aim_b4;
	uint8_t aim_dot1;
    uint8_t segment7;/* for dualleds*/
	uint8_t segment8;
	uint8_t segment9;
	uint8_t segment10;
	uint8_t pwm0;/*bit 4: invert mode bit3..0 period divider*/
	uint8_t pwm4;/*bit 4: invert mode bit3..0 period divider*/
	uint8_t max_disp_state;
	uint8_t analin2_curve[17]; /* detailed curve or offset and calibration (when linear) */
	uint8_t analin2_cal; /* curve scale */
	uint8_t analin2_offset; /* nonnegative offset */
	uint8_t analin2_mode;	/* dot position bit2:0=0..7 (7=after last digit) bit7:3=0 reserved */
    uint8_t analin_limiter; /* see: limit_analog()*/
    uint8_t boost_fixrefdc;
    uint8_t boost_fixtarget;
    /* the following 7 byte must not change order, refer to pwm_config_t structure*/
	uint8_t pwm_out0_control_kp; /* gas mixture lambda / boost control P term of PI */
	uint8_t pwm_out0_control_ki; /* gas mixture lambda / boost control I term of PI */
	uint8_t pwm_out0_control_kd; /* gas mixture lambda / boost control I term of PI */
	uint8_t pwm_out0_control_ilimit; /* gas mixture lambda / boost control pid integral limit */
	uint8_t pwm_out0_control_mode; /* bit 0..1: strategy: 00:lambda control, 01: boost control , 10: idle control */
	uint8_t pwm_out0_control_fixrefdc;
	uint8_t pwm_out0_control_fixtarget;
    uint8_t pwm_out0_control_mindc;
    uint8_t pwm_out0_control_maxdc;
    /* the following 7 byte must not change order, refer to pwm_config_t structure*/
	uint8_t pwm_out4_control_kp; /* gas mixture lambda / boost control P term of PI */
	uint8_t pwm_out4_control_ki; /* gas mixture lambda / boost control I term of PI */
	uint8_t pwm_out4_control_kd; /* gas mixture lambda / boost control I term of PI */
	uint8_t pwm_out4_control_ilimit; /* gas mixture lambda / boost control pid integral limit */
	uint8_t pwm_out4_control_mode; /* bit 0..1: strategy: 00:lambda control, 01: boost control , 10: idle control */
	uint8_t pwm_out4_control_fixrefdc;
	uint8_t pwm_out4_control_fixtarget;
    uint8_t pwm_out4_control_mindc;
    uint8_t pwm_out4_control_maxdc;
    uint8_t boost_control_mindc;
    uint8_t boost_control_maxdc;
    uint8_t peakval_select[2];  /* Peak value selection bits for TOP, and BOTTOM display */
};

extern struct config_t config;

#endif	/* GLOBAL_H */


